import { Listener } from '../observer';
import { UUID } from '../uuid';
export declare namespace DatabaseEvent {
    interface Update<T> {
        value: T;
    }
    export interface BeforeUpdate<T> extends Update<T> {
        newValue: T;
    }
    export interface AfterUpdate<T> extends Update<T> {
    }
    export {};
}
export declare namespace DatabaseAccessResult {
    interface Access<T> {
        value: T;
    }
    export interface Create<T> extends Access<T> {
    }
    export interface Read<T> extends Access<T> {
    }
    export interface Update<T> extends Access<T> {
    }
    export interface Delete {
        id: string;
        success: boolean;
    }
    export {};
}
export interface DatabaseRecord {
    id: string;
}
export interface Database<T extends DatabaseRecord> {
    isWorking: boolean;
    create(newValue: Omit<T, 'id'>): Promise<DatabaseAccessResult.Create<T>>;
    read(id: string): Promise<DatabaseAccessResult.Read<T>>;
    update(id: string, newValue: Omit<Partial<T>, 'id'>): Promise<DatabaseAccessResult.Update<T>>;
    delete(id: string): Promise<DatabaseAccessResult.Delete>;
    visit(visitor: (record: T) => void): Promise<void>;
    onBeforeUpdate(listener: Listener<DatabaseEvent.BeforeUpdate<T>>): () => void;
    onAfterUpdate(listener: Listener<DatabaseEvent.AfterUpdate<T>>): () => void;
}
export declare class InMemoryDatabase<T extends DatabaseRecord> implements Database<T> {
    private uuid;
    isWorking: boolean;
    private records;
    private beforeUpdateListeners;
    private afterUpdateListeners;
    constructor(uuid: UUID);
    create(newValue: Omit<T, 'id'>): Promise<DatabaseAccessResult.Create<T>>;
    read(id: string): Promise<DatabaseAccessResult.Read<T>>;
    update(id: string, newValue: Omit<Partial<T>, 'id'>): Promise<DatabaseAccessResult.Update<T>>;
    delete(id: string): Promise<DatabaseAccessResult.Delete>;
    visit(visitor: (record: T) => void): Promise<void>;
    onBeforeUpdate(listener: Listener<DatabaseEvent.BeforeUpdate<T>>): () => void;
    onAfterUpdate(listener: Listener<DatabaseEvent.AfterUpdate<T>>): () => void;
}
//# sourceMappingURL=inMemoryDatabase.d.ts.map