import { HTMLBindingDirective, } from '@microsoft/fast-element';
export const defaultEventMap = new Map([
    ['FAST-TEXT-AREA', 'input'],
    ['FAST-TEXT-FIELD', 'input'],
    ['FOUNDATION-TEXT-AREA', 'input'],
    ['FOUNDATION-TEXT-FIELD', 'input'],
    ['INPUT', 'input'],
    ['ZERO-TEXT-FIELD', 'input'],
]);
function customUpdateTarget(value) {
    if (this['keyAttr']) {
    }
    else {
        this.target[this.targetName] = value;
    }
}
function bind(source, context) {
    if (this['keyAttr']) {
        this.updateTarget = customUpdateTarget;
    }
    this.originalBind.call(this, source, context);
    this.target.addEventListener(this.changeEvent, this.changeHandler);
}
function unbind() {
    this.originalUnbind.call(this);
    this.target.removeEventListener(this.changeEvent, this.changeHandler);
}
function determineChangeEvent(target, eventName) {
    var _a;
    const tag = target.tagName;
    if (eventName === 'default') {
        return (_a = defaultEventMap.get(tag)) !== null && _a !== void 0 ? _a : 'change';
    }
    return eventName;
}
class SyncDirective extends HTMLBindingDirective {
    constructor(binding, convert, eventName, keyAttr) {
        super(binding);
        this.keyAttr = keyAttr;
        this.conversionType = convert;
        this.eventName = eventName;
    }
    createBehavior(target) {
        const behavior = super.createBehavior(target);
        behavior.originalBind = behavior.bind;
        behavior.originalUnbind = behavior.unbind;
        behavior.keyAttr = this.keyAttr;
        behavior.bind = bind;
        behavior.unbind = unbind;
        behavior.changeEvent = determineChangeEvent(target, this.eventName);
        behavior.changeHandler = () => {
            const value = this.conversionType === 'boolean' ? behavior.target.checked : behavior.target.value;
            if (this.keyAttr) {
                const last = behavior.bindingObserver.last;
                const key = behavior.target[this.keyAttr];
                last.propertySource[last.propertyName][key] = this.convert(value);
            }
            else {
                const last = behavior.bindingObserver.last;
                last.propertySource[last.propertyName] = this.convert(value);
            }
        };
        return behavior;
    }
    convert(value) {
        if (value === undefined || value === null)
            return value;
        switch (this.conversionType) {
            case 'time':
                const msSinceEpoch = new Date(value).getTime();
                return isNaN(msSinceEpoch) ? undefined : msSinceEpoch;
            case 'number':
                return value.length ? Number(value) : undefined;
            case 'string':
                return String(value);
            case 'boolean':
                return Boolean(value);
            default:
                return value;
        }
    }
}
export function sync(binding, conversionType = 'string', eventName = 'default', keyAttr) {
    return new SyncDirective(binding, conversionType, eventName, keyAttr);
}
//# sourceMappingURL=sync.js.map