import { __awaiter, __decorate } from "tslib";
import { observable } from '@microsoft/fast-element';
import { logger } from '../../utils';
export var PendingStateEvents;
(function (PendingStateEvents) {
    PendingStateEvents["state"] = "pending-state";
    PendingStateEvents["error"] = "pending-state-error";
})(PendingStateEvents || (PendingStateEvents = {}));
export const PendingState = (Base) => {
    class Pending extends Base {
        constructor() {
            super(...arguments);
            this.pendingCount = 0;
            this.resolvedCount = 0;
            this.hasPendingChildren = false;
        }
        get progress() {
            return this.resolvedCount / this.pendingCount;
        }
        connectedCallback() {
            super.connectedCallback();
            logger.debug(`PendingState mixin is now connected to the DOM`);
            this.addEventListener(PendingStateEvents.state, this.onPendingState);
        }
        disconnectedCallback() {
            super.disconnectedCallback();
            logger.debug(`PendingState mixin is now disconnected from the DOM`);
            this.removeEventListener(PendingStateEvents.state, this.onPendingState);
        }
        onPendingState({ detail }) {
            return __awaiter(this, void 0, void 0, function* () {
                const { promise } = detail;
                if (promise) {
                    this.hasPendingChildren = true;
                    this.pendingCount += 1;
                    try {
                        yield promise;
                    }
                    catch (error) {
                        this.$emit(PendingStateEvents.error, {
                            message: `Error in pending-state promise: ${error}`,
                            error,
                        });
                    }
                    finally {
                        this.resolvedCount += 1;
                        if (this.pendingCount === this.resolvedCount) {
                            this.hasPendingChildren = false;
                        }
                    }
                }
            });
        }
    }
    __decorate([
        observable
    ], Pending.prototype, "hasPendingChildren", void 0);
    return Pending;
};
//# sourceMappingURL=pendingState.js.map